<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	            xmlns:xs="http://www.w3.org/2001/XMLSchema"
              xmlns:psf="http://www.pageseeder.com/function"
              xmlns:ps="http://www.pageseeder.com/editing/2.0"
              exclude-result-prefixes="xs psf">
  <!--
  Templates used to produce Table of Contents (TOC) XML.
  
  @author  Willy Ekasalim
  @author Jean-Baptiste Reure
  @version 14 January 2011
  
  Copyright (C) 2007 Weborganic Systems Pty. Ltd.
  -->
  
  <!--
  changed, uses the recursive function psf:compute-labels now.
  
  @titles     The list of titles to consider as an array of nodes.
  @prefix     unused
  @level      unused
  @siteprefix unused
  @groupname  unused
  -->
  <xsl:function name="psf:toc-id-numbering">
    <xsl:param name="titles"/>
    <xsl:param name="numbering-config"/>
    
    <!-- process only when some titles are given: important otherwise it can lead to infinite recursing !!! -->
    <xsl:if test="count($titles[starts-with(name(), 'level')]) gt 0">
      <xsl:variable name="new-titles" select="psf:compute-labels($titles, 1)" />
      <!-- toc-container level="{$level}" -->
      <xsl:for-each select="$new-titles">
        <xsl:choose>
          <xsl:when test="starts-with(name(), 'level')">
            <xsl:element name="ps:{name()}">
              <xsl:attribute name="type" select="@type"/>
              <xsl:if test="@numbered"><xsl:attribute name="numbered" select="@numbered"/></xsl:if>
              <xsl:if test="@excluded"><xsl:attribute name="excluded" select="@excluded"/></xsl:if>
              <xsl:if test="@comment"><xsl:attribute name="comment" select="@comment"/></xsl:if>
              <!-- Count the number of heading previously, to be used for link within page-->
              <xsl:attribute name="idref" select="concat('toc-', count(preceding-sibling::*[starts-with(name(),'level')]))"/>
              <xsl:if test="@numbered = 'true'">
                <xsl:attribute name="canonical" select="@label"/>
                <xsl:attribute name="label">
                  <xsl:if test="@type = 'para' and $numbering-config//prefix-para = 'true'">
                    <xsl:variable name="level" select="count(tokenize(preceding-sibling::*[@type='heading'][1]/@label, '\.')[.])" />
                    <xsl:variable name="scheme" select="$numbering-config//heading-schemes/scheme[@level = $level]" />
                    <xsl:value-of select="psf:apply-scheme(preceding-sibling::*[@type='heading'][1]/@label, $numbering-config//strip-zeros = 'true', $scheme)" />
                  </xsl:if>
                  <xsl:variable name="level" select="count(tokenize(@label, '\.')[.])" />
                  <xsl:variable name="scheme" select="if (@type = 'para') then $numbering-config//para-schemes/scheme[@level = $level]
                                                                          else $numbering-config//heading-schemes/scheme[@level = $level]" />
                  <xsl:value-of select="psf:apply-scheme(@label, $numbering-config//strip-zeros = 'true', $scheme)"/>
                </xsl:attribute>
              </xsl:if>
              <xsl:apply-templates select="node()"/>
            </xsl:element>
          </xsl:when>
          <xsl:otherwise>
            <ps:xref>
              <xsl:attribute name="href" select="@href"/>
              <xsl:value-of select="."/>
            </ps:xref>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </xsl:if>
  </xsl:function>
  
  <xsl:variable name="alphabet" select="tokenize('a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z', ',')" />
  <xsl:variable name="romans" select="tokenize('i,ii,iii,iv,v,vi,vii,viii,ix,x,xi,xii,xiii,xiv,xv,xvi,xvii,xviii,xix,xx,xxi,xxii,xxiii,xxiv,xv,xvi', ',')" />
  
  <xsl:function name="psf:apply-scheme">
    <xsl:param name="canonical"/>
    <xsl:param name="strip-zeros" />
    <xsl:param name="scheme" />
    <xsl:if test="$scheme != ''">
      <xsl:analyze-string regex="\[.*?\]" select="$scheme">
        <xsl:matching-substring>
          <xsl:variable name="letter" select="replace(., '^\[(.*?)([a|A|i|I|1])(\d)(.*?)\]$', '$2')" />
          <xsl:variable name="level"  select="replace(., '^\[(.*?)([a|A|i|I|1])(\d)(.*?)\]$', '$3')" />
          <xsl:variable name="before" select="replace(., '^\[(.*?)([a|A|i|I|1])(\d)(.*?)\]$', '$1')" />
          <xsl:variable name="after"  select="replace(., '^\[(.*?)([a|A|i|I|1])(\d)(.*?)\]$', '$4')" />
          <xsl:variable name="value"  select="number(tokenize($canonical, '\.')[.][number($level)])" />
          <xsl:if test="$value != 0 or not($strip-zeros)"><xsl:value-of select="$before" /></xsl:if>
          <xsl:choose>
            <xsl:when test="$value = 0 and $strip-zeros" />
            <xsl:when test="$value = 0">0</xsl:when>
            <xsl:when test="$letter = '1'"><xsl:value-of select="$value" /></xsl:when>
            <xsl:when test="$letter = 'a'"><xsl:value-of select="$alphabet[$value]" /></xsl:when>
            <xsl:when test="$letter = 'A'"><xsl:value-of select="upper-case($alphabet[$value])" /></xsl:when>
            <xsl:when test="$letter = 'i'"><xsl:value-of select="$romans[$value]" /></xsl:when>
            <xsl:when test="$letter = 'I'"><xsl:value-of select="upper-case($romans[$value])" /></xsl:when>
          </xsl:choose>
          <xsl:if test="$value != 0 or not($strip-zeros)"><xsl:value-of select="$after" /></xsl:if>
        </xsl:matching-substring>
      </xsl:analyze-string>
    </xsl:if>
  </xsl:function>
  
  <!--
  Removes zeros from a label
  
  @label      numeric label (e.g. 1.0.2.3)
  -->
  <xsl:function name="psf:remove-zeros">
    <xsl:param name="label"/>   
    <xsl:choose>
    <xsl:when test="contains($label,'0.')">
	    <xsl:for-each select="tokenize($label,'\.')">
	    	<xsl:if test=".!='0'">
		    	<xsl:value-of select="."/>
		    	<xsl:if test="position()!=last()">.</xsl:if>
		    </xsl:if>
	    </xsl:for-each>
    </xsl:when>
    <xsl:otherwise>
		<xsl:value-of select="$label"/>	    	 
    </xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <!--
  A recursive 'local' function which adds the label attribute to all titles
  
  @titles     The list of titles to consider as an array of nodes.
  @position   The current position
  -->
  <xsl:function name="psf:compute-labels">
    <xsl:param name="titles"/>
    <xsl:param name="position"/>
    
    <xsl:variable name="new-titles">
      <xsl:for-each select="$titles">
        <xsl:choose>
          <!-- Rules for numbered paragraphs -->
          <xsl:when test="position() = $position and @numbered = 'true' and starts-with(name(), 'level') and @type = 'para'">
            <xsl:copy>
              <xsl:copy-of select="@*[name() != 'label']" />
              <xsl:variable name="current-level" select="number(substring-after(name(.), 'level'))" />
              <xsl:variable name="previous-heading" select="preceding-sibling::*[@type='heading'][@numbered = 'true'][@label][1]" />
              <xsl:variable name="significant-previous"
                            select="preceding-sibling::*[@numbered = 'true'][@label][@type = 'para']
                                                        [preceding-sibling::*[generate-id() = generate-id($previous-heading)] or not($previous-heading)]
                                                        [number(substring-after(name(), 'level')) lt $current-level][1]" />
              <xsl:variable name="prev-count" select="count(preceding-sibling::*[name() = current()/name()][@numbered = 'true'][@type='para']
                                  [preceding-sibling::*[generate-id() = generate-id($previous-heading)] or not($previous-heading)]
                                  [empty($significant-previous) or preceding-sibling::*[generate-id() = generate-id($significant-previous)]])"/>
              <xsl:variable name="start-property" select="$titles//property[@name=concat('autonumber-start-level',$current-level)]"/>
              <xsl:variable name="start-count" select="if (empty($significant-previous) and $start-property)
                                  then (if (preceding-sibling::*[@numbered = 'true']) then number($start-property) else number($start-property)-1)
                                  else 0"/>
              <xsl:variable name="prev-level" select="if ($significant-previous) then 
                                  number(substring-after(name($significant-previous), 'level')) else 0" />
              <xsl:variable name="prev-label">
                <xsl:choose>
                  <xsl:when test="empty($significant-previous)">
                    <xsl:for-each select="1 to (($current-level - 1) cast as xs:integer)">
                      <xsl:variable name="start-prop" select="$titles//property[@name=concat('autonumber-start-level',current())]"/>
                      <xsl:value-of select="if ($start-prop) then $start-prop else '0'"/>
                      <xsl:text>.</xsl:text>
                    </xsl:for-each>
                  </xsl:when>
                  <xsl:when test="$current-level = 1"></xsl:when>
                  <xsl:when test="$current-level = $prev-level + 1 and $significant-previous/@label">
                    <xsl:value-of select="$significant-previous/@label" />
                  </xsl:when>
                  <!-- <xsl:when test="$current-level = $prev-level + 1">0.</xsl:when> -->
                  <xsl:when test="$current-level gt $prev-level + 1 and $significant-previous/@label">
                    <xsl:value-of select="concat($significant-previous/@label, string-join(
                                          for $i in 1 to (($current-level - $prev-level - 1) cast as xs:integer) return '0.'
                                          , ''))" />
                  </xsl:when>
                  <!-- 
                  <xsl:when test="$current-level gt $prev-level + 1">
                    <xsl:value-of select="string-join(
                      for $i in 1 to (($current-level - $prev-level - 1) cast as xs:integer) return '0.'
                      , '')" />
                  </xsl:when>
                   -->
                  <xsl:otherwise>unknown</xsl:otherwise>
                </xsl:choose>
              </xsl:variable>
              <xsl:attribute name="label" select="concat($prev-label, $start-count + $prev-count + 1, '.')" />
              <xsl:copy-of select="node()" />
            </xsl:copy>
          </xsl:when>
          <!-- Rules for rest: numbered headings -->
          <xsl:when test="position() = $position and @numbered = 'true' and starts-with(name(), 'level')">
            <xsl:copy>
              <xsl:copy-of select="@*[name() != 'label']" />
              <xsl:variable name="current-level" select="number(substring-after(name(.), 'level'))" />
              <xsl:variable name="significant-previous"
                            select="preceding-sibling::*[@numbered = 'true'][@label][@type != 'para']
                                                        [number(substring-after(name(), 'level')) lt $current-level][1]" />
              <xsl:variable name="prev-count" select="count(preceding-sibling::*[name() = current()/name()][@numbered = 'true'][@type != 'para']
                                  [empty($significant-previous) or preceding-sibling::*[generate-id() = generate-id($significant-previous)]])"/>
      			  <xsl:variable name="start-property" select="$titles//property[@name=concat('autonumber-start-level',$current-level)]"/>
      			  <xsl:variable name="start-count" select="if (empty($significant-previous) and $start-property)
                        			  	then (if (preceding-sibling::*[@numbered = 'true']) then number($start-property) else number($start-property)-1)
                        			  	else 0"/>
              <xsl:variable name="prev-level" select="if ($significant-previous) then 
                                  number(substring-after(name($significant-previous), 'level')) else 0" />
              <xsl:variable name="prev-label">
                <xsl:choose>
                  <xsl:when test="empty($significant-previous)">
                  	<xsl:for-each select="1 to (($current-level - 1) cast as xs:integer)">
			  			        <xsl:variable name="start-prop" select="$titles//property[@name=concat('autonumber-start-level',current())]"/>
                  		<xsl:value-of select="if ($start-prop) then $start-prop else '0'"/>
                  		<xsl:text>.</xsl:text>
                  	</xsl:for-each>
                  </xsl:when>
                  <xsl:when test="$current-level = 1"></xsl:when>
                  <xsl:when test="$current-level = $prev-level + 1 and $significant-previous/@label">
                    <xsl:value-of select="$significant-previous/@label" />
                  </xsl:when>
                  <!-- <xsl:when test="$current-level = $prev-level + 1">0.</xsl:when> -->
                  <xsl:when test="$current-level gt $prev-level + 1 and $significant-previous/@label">
                    <xsl:value-of select="concat($significant-previous/@label, string-join(
                                          for $i in 1 to (($current-level - $prev-level - 1) cast as xs:integer) return '0.'
                                          , ''))" />
                  </xsl:when>
                  <!-- 
                  <xsl:when test="$current-level gt $prev-level + 1">
                    <xsl:value-of select="string-join(
                      for $i in 1 to (($current-level - $prev-level - 1) cast as xs:integer) return '0.'
                      , '')" />
                  </xsl:when>
                   -->
                  <xsl:otherwise>unknown</xsl:otherwise>
                </xsl:choose>
              </xsl:variable>
              <xsl:attribute name="label" select="concat($prev-label, $start-count + $prev-count + 1, '.')" />
              <xsl:copy-of select="node()" />
            </xsl:copy>
          </xsl:when>
          <xsl:otherwise><xsl:sequence select="." /></xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$titles[$position+1]">
        <xsl:sequence select="psf:compute-labels($new-titles/*, $position+1)" />
      </xsl:when>
      <xsl:otherwise>
        <xsl:sequence select="$new-titles/*" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <!--
  Generates the Layout-style XRef URL.
  If href starts with # just return href
  
  @param  site-prefix  PageSeeder site prefix
  @param  group-name   current group name
  @param  href         href attribute from xref
  -->
  <xsl:function name="psf:to-xref-path">
    <xsl:param name="site-prefix"/>
    <xsl:param name="group-name"/>
    <xsl:param name="href"/>
    
    <xsl:choose>
      <xsl:when test="starts-with($href,'#')">
        <xsl:value-of select="$href"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of
          select="concat($site-prefix,'/page/',$group-name,psf:to-document-path($site-prefix, $group-name, $href))"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <!--
  Generates the Layout-style path URL
  
  @param  site-prefix  PageSeeder site prefix
  @param  group-name   current group name
  @param  ps-path      uri path for document
  -->
  <xsl:function name="psf:to-document-path">
    <xsl:param name="site-prefix"/>
    <xsl:param name="group-name"/>
    <xsl:param name="ps-path"/>
    <!-- prefix of files in the same group, eg. ps/my/group -->
    <xsl:variable name="group-prefix" select="concat($site-prefix, '/' ,translate($group-name, '-', '/'))"/>
    <xsl:value-of
      select="
    if (starts-with($ps-path, $group-prefix))
      then
        concat('/path', substring($ps-path, string-length($group-prefix)+1))
      else
        concat('/fullpath', substring($ps-path, string-length($site-prefix)+1))
  "/>
  </xsl:function>
  
</xsl:stylesheet>