<!-- 
  This template splits an Excel spreadsheet into a document per row so that it can be fed into a template.
  
  @author Christophe Lauret
  @version 27 October 2011
-->
<xsl:stylesheet version="2.0"
      xmlns:ss="http://schemas.openxmlformats.org/spreadsheetml/2006/main"
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
      xmlns:xs="http://www.w3.org/2001/XMLSchema"
      exclude-result-prefixes="xsl xs ss">

  <!-- Which column to use as the filename? -->
  <xsl:param name="filename-col" select="'0'"/> 

  <!-- Which column to use as the filename? -->
  <xsl:param name="first-row-is-header" select="'true'"/>

  <xsl:output encoding="UTF-8" method="xml" indent="yes" />

  <!-- folder -->
  <xsl:variable name="folder" select="string-join(tokenize(base-uri(), '/')[not(position() = last())], '/')"/>

  <!-- Load shared strings -->
  <xsl:variable name="sharedstrings" select="document(concat($folder, '/sharedStrings.xml'))"/>

  <!-- Process the workbook -->
  <xsl:template match="ss:workbook">
    <xsl:for-each select="ss:sheets/ss:sheet"> 
      <xsl:sort select="@sheetId"/>
      <xsl:variable name="title" select="@name"/>
        <xsl:apply-templates select="document(concat($folder,'/worksheets/sheet', @sheetId, '.xml'))">
          <xsl:with-param name="title" select="$title"/>
        </xsl:apply-templates>        
    </xsl:for-each>
  </xsl:template>

  <!-- Process a work sheet -->
  <xsl:template match="ss:worksheet">
    <xsl:param name="title"/>
    <!-- Grab the titles from the first row -->
    <xsl:variable name="col-titles">
      <head>
        <xsl:for-each select="ss:sheetData/ss:row[1]/ss:c">
          <col><xsl:value-of select="ss:get-value(.)"/></col>
        </xsl:for-each>
      </head>
    </xsl:variable>
    <xsl:variable name="fncol" select="xs:integer($filename-col)"/>
    <xsl:for-each select="ss:sheetData/ss:row[if ($first-row-is-header = 'true') then position() gt 1 else position() ge 1]">
      <xsl:variable name="filename" select="if ($fncol gt 0 and $fncol le count(ss:c))
                                              then ss:get-value(ss:c[$fncol])
                                              else format-number(position(), '0000')"/>
      <xsl:result-document href="{substring-before(tokenize(base-uri(), '/')[last()], '.')}/{$filename}.xml" method="xml" encoding="utf-8" indent="yes">
      <row position="{@r}" sheet-title="{$title}">
        <xsl:for-each select="ss:c">
          <col>
            <xsl:if test="$first-row-is-header = 'true'">
              <xsl:variable name="p" select="position()"/>
              <xsl:attribute name="title" select="$col-titles//col[$p]"/>
            </xsl:if>
            <xsl:value-of select="ss:get-value(.)"/>
          </col>
        </xsl:for-each>
      </row>
      </xsl:result-document>
    </xsl:for-each>
  </xsl:template>

  <xsl:function name="ss:get-value">
    <xsl:param name="c"/>
    <xsl:value-of select="if ($c/@t = 's') then ss:get-share-string($c/ss:v) else $c/ss:v"/>
  </xsl:function>

  <xsl:function name="ss:get-share-string">
    <xsl:param name="i"/>
    <xsl:value-of select="$sharedstrings//ss:si[$i + 1]/ss:t"/>
  </xsl:function>

</xsl:stylesheet>

