<?xml version="1.0"?>
<!--
  Used to generate a set of documents ready for import.
  The graphics and XRefs are modified using the parameters specified:
  - preserve-external-xrefs [default:true]
  - preserve-internal-xrefs [default:true]
  - preserve-external-graphics [default:true]
  - preserve-internal-graphics [default:true]
  - label-removed-xrefs [default:false]
  - label-removed-graphics [default:false]
  - preserve-labels [default:true]
  - preserve-titles [default:true]
  - preserve-docids [default:true]

  @author Jean-Baptiste Reure
  @version 10 April 2012

  Copyright (C) 2009 Weborganic Systems Pty. Ltd.
-->
<xsl:stylesheet version="2.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:psf="http://www.pageseeder.com/function"
                exclude-result-prefixes="xs psf">

  <xsl:output method="text"/>
  
  <xsl:param name="input-base" />
  <xsl:param name="output-base" />
  <xsl:param name="preserve-external-xrefs">true</xsl:param>
  <xsl:param name="preserve-internal-xrefs">true</xsl:param>
  <xsl:param name="preserve-external-graphics">true</xsl:param>
  <xsl:param name="preserve-internal-graphics">true</xsl:param>
  <xsl:param name="label-removed-xrefs">false</xsl:param>
  <xsl:param name="label-removed-graphics">false</xsl:param>
  <xsl:param name="preserve-labels">true</xsl:param>
  <xsl:param name="preserve-titles">true</xsl:param>
  <xsl:param name="preserve-docids">true</xsl:param>
  
  <xsl:variable name="uris" select="/uris/uri" />
  
  <xsl:template match="/uris">
    <xsl:if test="$preserve-internal-xrefs = 'true' or $preserve-internal-graphics = 'true' or
                  $preserve-external-xrefs = 'false' or $preserve-external-graphics = 'false' or
                  $preserve-labels = 'true' or $preserve-docids = 'true' or $preserve-titles = 'true'">
      <xsl:for-each select="uri[starts-with(behavior, 'standard')]">
        <xsl:variable name="doc" select="document(concat(replace($input-base, '\\', '/'), '/', decodedpath))" />
        <xsl:variable name="new-url" select="concat(replace($output-base, '\\', '/'), '/', decodedpath)" />
        <xsl:result-document href="{$new-url}" method="xml" indent="no">
          <xsl:apply-templates select="$doc">
            <xsl:with-param name="uri"    select="current()" tunnel="yes" />
            <xsl:with-param name="folder" select="concat(string-join(tokenize(current()/path, '/')[position() != last()], '/'), '/')" tunnel="yes" />
          </xsl:apply-templates>
        </xsl:result-document>
        <xsl:value-of select="$new-url" /><xsl:text>
</xsl:text>
      </xsl:for-each>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="graphic">
    <xsl:param name="uri"    tunnel="yes" />
    <xsl:param name="folder" tunnel="yes" />
    <xsl:variable name="turi" select="substring-after(@src, '/ps/uri/')" />
    <xsl:variable name="target-uri" select="if (contains($turi, '.')) then substring-before($turi, '.') else $turi" />
    <xsl:variable name="within" select="not(starts-with(@src,'/')) or not(empty($uris[id = $target-uri]))" />
    <xsl:choose>
      <!-- target is within the export and we preserve the internal graphics -->
      <xsl:when test="$within and $preserve-internal-graphics = 'true'">
        <xsl:copy>
          <xsl:copy-of select="@*[name() != 'src']" />
          <xsl:attribute name="src" select="if (not(starts-with(@src,'/'))) then @src else psf:compute-relative-path($uris[id = $target-uri]/path, $folder)" />
          <xsl:apply-templates />
        </xsl:copy>
      </xsl:when>
      <!-- target is within the export and we label removed graphics -->
      <xsl:when test="$within and $label-removed-graphics = 'true'">
        <xsl:element name="{if (parent::body) then 'paraLabel' else 'inlineLabel'}">
          <xsl:attribute name="name">removed-graphic</xsl:attribute>
          <xsl:value-of select="if (not(starts-with(@src,'/'))) then @src else $uris[id = $target-uri]/path" />
        </xsl:element>
      </xsl:when>
      <!-- target is within the export and we DO NOT preserve the internal graphics -->
      <xsl:when test="$within" />
      <!-- target is outside of the export and we preserve the external graphics -->
      <xsl:when test="$preserve-external-graphics = 'true'">
        <xsl:copy><xsl:copy-of select="@*" /><xsl:apply-templates /></xsl:copy>
      </xsl:when>
      <!-- target is outside of the export and we label removed graphics -->
      <xsl:when test="$label-removed-graphics = 'true'">
        <xsl:element name="{if (parent::body) then 'paraLabel' else 'inlineLabel'}">
          <xsl:attribute name="name">removed-graphic</xsl:attribute>
          <xsl:value-of select="@src" />
        </xsl:element>
      </xsl:when>
      <!-- target is within the export and we DO NOT preserve the internal graphics -->
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="xref | blockXref">
    <xsl:param name="folder" tunnel="yes" />
    
    <xsl:choose>
      <!-- target is within the export and we preserve the internal xrefs -->
      <!-- Remove docid attribute as the href attribute is enough to point to the correct target -->
      <xsl:when test="not(empty($uris[id = current()/@uriid])) and $preserve-internal-xrefs = 'true'">
        <xsl:copy>
          <xsl:copy-of select="@*[name() != 'path'][name() != 'uriid'][name() != 'href'][name() != 'docid']" />
          <xsl:attribute name="href" select="psf:compute-relative-path($uris[id = current()/@uriid]/path, $folder)" />
          <xsl:apply-templates />
        </xsl:copy>
      </xsl:when>
      <!-- target is within the export and we label removed xrefs -->
      <xsl:when test="not(empty($uris[id = current()/@uriid])) and $label-removed-xrefs = 'true'">
        <xsl:element name="{if (self::blockXref) then 'paraLabel' else 'inlineLabel'}">
          <xsl:attribute name="name">removed-xref</xsl:attribute>
          <xsl:value-of select="$uris[id = current()/@uriid]/path" />
        </xsl:element>
      </xsl:when>
      <!-- target is within the export and we DO NOT preserve the internal xrefs -->
      <xsl:when test="not(empty($uris[id = current()/@uriid]))"><xsl:value-of select="." /></xsl:when>
      <!-- target is outside of the export and we preserve the external xrefs -->
      <xsl:when test="$preserve-external-xrefs = 'true'">
        <xsl:copy><xsl:copy-of select="@*[name() != 'uriid'][name() != 'path']" /><xsl:apply-templates /></xsl:copy>
      </xsl:when>
      <!-- target is outside of the export and we label removed xrefs -->
      <xsl:when test="$label-removed-xrefs = 'true'">
        <xsl:element name="{if (self::blockXref) then 'paraLabel' else 'inlineLabel'}">
          <xsl:attribute name="name">removed-xref</xsl:attribute>
          <xsl:value-of select="@href" />
        </xsl:element>
      </xsl:when>
      <!-- target is outside of the export and we DO NOT preserve the external xrefs -->
      <xsl:otherwise><xsl:value-of select="." /></xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="*">
    <xsl:copy>
      <xsl:copy-of select="@*" />
      <xsl:apply-templates />
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="processing-instruction()">
    <xsl:param name="uri" tunnel="yes" />
    <xsl:processing-instruction name="{name()}">
      <xsl:value-of select="psf:replace-labels(psf:replace-title(psf:replace-docid(., $uri), $uri), $uri)" />
    </xsl:processing-instruction>
  </xsl:template>
  
  <xsl:function name="psf:compute-relative-path" as="xs:string">
    <xsl:param name="fullpath" />
    <xsl:param name="base" />
    <xsl:choose>
      <xsl:when test="starts-with($fullpath, $base)">
        <xsl:value-of select="substring-after($fullpath, $base)" />
      </xsl:when>
      <xsl:otherwise>
        <!-- example: fullpath = '/a/b/e.xml' and base is '/a/b/c/d' -->
        <!-- get all folders in base path
             example: folders are 'a', 'b', 'c' and 'd' -->
        <xsl:variable name="all-folders" select="tokenize($base, '/')[.]" />
        <!-- get all folders in base path that match the current link
             example: matching folders are 'a' and 'b' -->
        <xsl:variable name="matching-subfolders">
          <xsl:for-each select="for $i in 1 to count($all-folders) return concat('/', string-join($all-folders[position() le $i], '/'), '/')">
            <xsl:if test=". != '/' and starts-with($fullpath, .)"><sub xmlns=""><xsl:value-of select="." /></sub></xsl:if>
          </xsl:for-each>
        </xsl:variable>
        <!-- all folders that don't match the link are replaced with '../' and the others are removed
             example: non matching folders are 'c' and 'd' so first part of link '../../'
                      and matching folders are 'a' and 'b' so second part of link is 'e.xml' -->
        <xsl:value-of select="concat(string-join(for $i in 1+count($matching-subfolders//sub) to count($all-folders) return '../', ''),
                                     substring-after($fullpath, $matching-subfolders//sub[last()]))" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="psf:replace-labels" as="xs:string">
    <xsl:param name="original" />
    <xsl:param name="uri" />
    <xsl:choose>
      <xsl:when test="$preserve-labels = 'true'">
        <xsl:variable name="labels" select="if ($uri/labels) then concat(' labels=&quot;', $uri/labels, '&quot;') else ''" />
        <xsl:value-of select="concat(replace($original, 'labels=&quot;.*?&quot;', ''), $labels)" />
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$original" /></xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="psf:replace-docid" as="xs:string">
    <xsl:param name="original" />
    <xsl:param name="uri" />
    <xsl:choose>
      <xsl:when test="$preserve-docids = 'true'">
        <xsl:variable name="docid" select="if ($uri/documentid) then concat(' docid=&quot;', $uri/documentid, '&quot;') else ''" />
        <xsl:value-of select="concat(replace($original, 'docid=&quot;;.*?&quot;', ''), $docid)" />
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$original" /></xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="psf:replace-title" as="xs:string">
    <xsl:param name="original" />
    <xsl:param name="uri" />
    <xsl:choose>
      <xsl:when test="$preserve-titles = 'true'">
        <xsl:variable name="title" select="if ($uri/usertitle) then concat(' title=&quot;', $uri/usertitle, '&quot;') else ''" />
        <xsl:value-of select="concat(replace($original, 'title=&quot;.*?&quot;', ''), $title)" />
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$original" /></xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
</xsl:stylesheet>