<!--
  A collection of function to handle media types

  This XSLT is designed for reuse, it should:
  - only contains functions and;
  - not declare any global variable and;
  - not import or include any other XSLT.

  @author  Christophe Lauret
  @version 5.7902
  @since   5.7902
-->
<xsl:stylesheet version="2.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:psf="http://www.pageseeder.com/function"
                exclude-result-prefixes="#all">

<!--
  Returns the small media type icon to use for this URI.

  @param uri a URI element

  @returns the icon in an 'i' element.
-->
<xsl:function name="psf:to-mediaicon" as="element(i)">
  <xsl:param name="uri" as="element(uri)"/>
  <xsl:sequence select="psf:uri-to-mediaicon($uri, 16)"/>
</xsl:function>

<!--
  Returns the small media type icon from an index document.

  @param document a document element from the index

  @returns the corresponding icon in an 'i' element.
-->
<xsl:function name="psf:document-to-mediaicon" as="element(i)">
  <xsl:param name="document" as="element(document)"/>
  <xsl:sequence select="psf:document-to-mediaicon($document, 16)"/>
</xsl:function>

<!--
  Returns the small media type icon from a file

  @param file a file element

  @returns the corresponding in an 'i' element.
-->
<xsl:function name="psf:file-to-mediaicon" as="element(i)">
  <xsl:param name="file" as="element(file)"/>
  <xsl:sequence select="psf:file-to-mediaicon($file, 16)"/>
</xsl:function>

<!--
  Return the name corresponding to the specified media type

  @param mediatype The mediatype of the specified file.
  @return the corresponding name
-->
<xsl:function name="psf:to-mediatype-name">
  <xsl:param name="mediatype"/>
  <xsl:choose>
    <xsl:when test="$mediatype = 'application/vnd.pageseeder.url+xml'">PageSeeder URL</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.pageseeder.comment+xml'">PageSeeder comment</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.pageseeder.task+xml'">PageSeeder task</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.pageseeder.document.standard+xml'">PS standard document</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.pageseeder.psml+xml'">PSML Document</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'">Word document</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'">Excel spreadsheet</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.openxmlformats-officedocument.presentationml.presentation'">PowerPoint presentation</xsl:when>
    <xsl:when test="$mediatype = 'application/msword'">Word 97-2003 document</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.ms-excel'">Excel 97-2003 spreadsheet</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.ms-powerpoint'">PowerPoint 97-2003 presentation</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.ms-xpsdocument'">XPS document</xsl:when>
    <xsl:when test="$mediatype = 'application/pdf'">PDF document</xsl:when>
    <xsl:when test="$mediatype = 'application/zip'">ZIP file</xsl:when>
    <xsl:when test="$mediatype = 'application/x-gzip'">GZIP file</xsl:when>
    <xsl:when test="$mediatype = 'application/x-tar'">Tape archive file</xsl:when>
    <xsl:when test="$mediatype = 'application/xml'">XML document</xsl:when>
    <xsl:when test="$mediatype = 'application/illustrator'">Illustrator document</xsl:when>
    <xsl:when test="$mediatype = 'application/javascript'">JavaScript file</xsl:when>
    <xsl:when test="$mediatype = 'application/json'">JSON file</xsl:when>
    <xsl:when test="$mediatype = 'application/x-shockwave-flash'">Flash document</xsl:when>
    <xsl:when test="$mediatype = 'application/x-actionscript'">ActionScript document</xsl:when>
    <xsl:when test="$mediatype = 'application/emf'">Enhanced meta document</xsl:when>
    <xsl:when test="$mediatype = 'application/x-msmetafile'">Windows meta document</xsl:when>
    <xsl:when test="$mediatype = 'application/x-font-ttf'">TrueType font</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.ms-fontobject'">OpenType font</xsl:when>
    <xsl:when test="$mediatype = 'application/font-woff'">Web Open font</xsl:when>
    <xsl:when test="$mediatype = 'application/rtf'">Rich Text Format document</xsl:when>
    <xsl:when test="$mediatype = 'application/xslt+xml'">XSLT stylesheet</xsl:when>
    <xsl:when test="$mediatype = 'text/css'">CSS document</xsl:when>
    <xsl:when test="$mediatype = 'text/csv'">CSV document</xsl:when>
    <xsl:when test="$mediatype = 'text/html'">HTML document</xsl:when>
    <xsl:when test="$mediatype = 'text/plain'">Text document</xsl:when>
    <xsl:when test="$mediatype = 'text/properties'">Properties file</xsl:when>
    <xsl:when test="$mediatype = 'text/xml'">XML document</xsl:when>
    <xsl:when test="$mediatype = 'audio/mpeg'">MPEG audio</xsl:when>
    <xsl:when test="$mediatype = 'image/jpeg'">JPG image</xsl:when>
    <xsl:when test="$mediatype = 'image/gif'">GIF image</xsl:when>
    <xsl:when test="$mediatype = 'image/png'">PNG image</xsl:when>
    <xsl:when test="$mediatype = 'image/bmp'">BitMap image</xsl:when>
    <xsl:when test="$mediatype = 'image/svg+xml'">SVG image</xsl:when>
    <xsl:when test="$mediatype = 'image/vnd.adobe.photoshop'">Photoshop image</xsl:when>
    <xsl:when test="$mediatype = 'video/mp4'">MPEG video</xsl:when>
    <xsl:when test="$mediatype = 'video/x-flv'">Flash video</xsl:when>
    <xsl:when test="starts-with($mediatype, 'video/')">Other video (<xsl:value-of select="substring-after($mediatype, 'video/')"/>)</xsl:when>
    <xsl:when test="starts-with($mediatype, 'audio/')">Other audio (<xsl:value-of select="substring-after($mediatype, 'audio/')"/>)</xsl:when>
    <xsl:when test="starts-with($mediatype, 'image/')">Other image (<xsl:value-of select="substring-after($mediatype, 'image/')"/>)</xsl:when>
    <xsl:when test="starts-with($mediatype, 'text/')">Other text (<xsl:value-of select="substring-after($mediatype, 'text/')"/>)</xsl:when>
    <xsl:when test="$mediatype = 'video/x-flv'">Flash video</xsl:when>
    <xsl:when test="$mediatype = 'application/octet-stream'">Flash video</xsl:when>
    <xsl:when test="$mediatype = ''">None</xsl:when>
    <xsl:otherwise>Other</xsl:otherwise>
  </xsl:choose>
</xsl:function>

<!--
  Returns the small media type icon to use for this URI.

  @param uri a URI element
  @param size in pixels (16 or 32)

  @returns the icon in an 'i' element.
-->
<xsl:function name="psf:uri-to-mediaicon" as="element(i)">
  <xsl:param name="uri" as="element(uri)"/>
  <xsl:param name="size" as="xs:integer"/>
  <xsl:choose>
    <!-- URLs -->
    <xsl:when test="$uri/@external = 'true' and $uri/@folder = 'true'">
      <i class="psi psi-{$size} psi-folder{if ($uri/@archived = 'true') then '-archive' else ''}"/>
    </xsl:when>
    <xsl:when test="$uri/@external = 'true' and $uri/@archived = 'true'">
      <i class="psi psi-{$size} psi-url-archive"/>
    </xsl:when>
    <xsl:when test="$uri/@external = 'true' and $uri/@scheme = 'https'">
      <i class="psi psi-{$size} psi-url-secure"/>
    </xsl:when>
    <xsl:when test="$uri/@external = 'true'">
      <i class="psi psi-{$size} psi-url"/>
    </xsl:when>
    <xsl:when test="$uri/type = 'folder'">
      <i class="psi psi-{$size} psi-folder{if (count($uri/sharing/share/group) gt 1) then '-shared' else ''}"/>
    </xsl:when>
    <xsl:when test="$uri/type">
      <i class="psi psi-{$size} psi-mediatype {psf:_to-mediaicon-class($uri/type, $uri/path, $uri/behavior)}"/>
    </xsl:when>
    <!-- TODO: Quirk for the activity report, remove when fixed -->
    <xsl:when test="$uri/@type">
      <i class="psi psi-{$size} psi-mediatype {psf:_to-mediaicon-class($uri/@type, $uri/@path, $uri/@behavior)}"/>
    </xsl:when>
    <!-- folder -->
    <xsl:when test="$uri/@mediatype = 'folder'">
      <i class="psi psi-{$size} psi-folder{if ($uri/@sharing = 'shared') then '-shared' else ''}"/>
    </xsl:when>
    <xsl:when test="$uri/@mediatype">
      <i class="psi psi-{$size} psi-mediatype {psf:_to-mediaicon-class($uri/@mediatype, $uri/@path, ())}"/>
    </xsl:when>
    <xsl:otherwise>
      <i class="psi psi-{$size} psi-mediatype"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:function>

<!--
  Returns the small media type icon from an index document.

  @param document a document element from the index
  @param size in pixels (16 or 32)

  @returns the corresponding icon in an 'i' element.
-->
<xsl:function name="psf:document-to-mediaicon" as="element(i)">
  <xsl:param name="document" as="element(document)"/>
  <xsl:param name="size" as="xs:integer"/>
  <xsl:choose>
    <xsl:when test="$document/field[@name='pstype'] = 'comment'"><i class="psi psi-{$size} psi-comment"/></xsl:when>
    <xsl:when test="$document/field[@name='pstype'] = 'task'"   ><i class="psi psi-{$size} psi-task"/></xsl:when>
    <xsl:when test="$document/field[@name='pstype'] = 'folder'" ><i class="psi psi-{$size} psi-folder"/></xsl:when>
    <xsl:when test="$document/field[@name='pstype'] = 'url'"    ><i class="psi psi-{$size} psi-url{if (starts-with($document/field[@name='pscontent'], 'https://')) then '-secure' else ''}"/></xsl:when>
    <xsl:otherwise>
      <i class="psi psi-{$size} psi-mediatype {psf:_to-mediaicon-class($document/field[@name='psmediatype'], $document/field[@name='psfilename'], ())}"
         title="{psf:to-mediatype-name($document/field[@name = 'psmediatype'])} [{$document/field[@name = 'psmediatype']}]"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:function>

<!--
  Returns the small media type icon from a file

  @param file a file element
  @param size in pixels (16 or 32)

  @returns the corresponding in an 'i' element.
-->
<xsl:function name="psf:file-to-mediaicon" as="element(i)">
  <xsl:param name="file" as="element(file)"/>
  <xsl:param name="size" as="xs:integer?"/>
  <i class="psi psi-{$size} psi-mediatype {if ($file/@format = 'standard') then 'psstandard' else
                                           if ($file/@media-type) then psf:_to-mediaicon-class($file/@media-type, $file/@path, ()) else ''}"/>
</xsl:function>

<!-- Functions below should remain private ==================================================== -->

<!--
  Return the document type from uri type and path.

  Note designed to be used directly.

  @param type     The media type
  @param path     The path to the URI
  @param behavior The behavior
-->
<xsl:function name="psf:_to-mediaicon-class" as="xs:string">
  <xsl:param name="mediatype" as="xs:string"/>
  <xsl:param name="path"      as="xs:string?"/>
  <xsl:param name="behavior"  as="xs:string?"/>
  <!-- The name of the file -->
  <xsl:variable name="ext" select="lower-case(tokenize($path, '\.')[last()])"/>
  <xsl:choose>
    <!-- PageSeeder types -->
    <xsl:when test="$behavior and starts-with($behavior, 'standard')">psstandard</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.pageseeder.document.standard+xml'">psstandard</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.pageseeder.psml+xml'">psml</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.pageseeder.url+xml'">url</xsl:when>
    <!-- Specific MIME -->
    <xsl:when test="$mediatype = 'application/pdf'">pdf</xsl:when>
    <xsl:when test="starts-with($mediatype, 'application/vnd.ms-word') or $mediatype = 'application/msword'">ms-word</xsl:when>
    <xsl:when test="starts-with($mediatype, 'application/vnd.ms-excel')">ms-excel</xsl:when>
    <xsl:when test="starts-with($mediatype, 'application/vnd.ms-powerpoint')">ms-powerpoint</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'">openxml-word</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'">openxml-excel</xsl:when>
    <xsl:when test="$mediatype = 'application/vnd.openxmlformats-officedocument.presentationml.presentation'">openxml-powerpoint</xsl:when>
    <xsl:when test="$mediatype = 'text/html'">html</xsl:when>
    <xsl:when test="$mediatype = 'text/css' or $ext='scss'">css</xsl:when>
    <xsl:when test="$mediatype = 'text/properties'">properties</xsl:when>
    <xsl:when test="$mediatype = 'application/javascript' or $mediatype='application/json'">javascript</xsl:when>
    <xsl:when test="$mediatype = 'application/zip' ">compressed</xsl:when>
    <xsl:when test="$mediatype = 'application/x-tar'">compressed</xsl:when>
    <xsl:when test="$mediatype = 'application/x-gzip'">compressed</xsl:when>
    <xsl:when test="$mediatype = 'application/java-archive'">compressed</xsl:when>
    <xsl:when test="$mediatype = 'application/xml-dtd' or $mediatype='application/schematron+xml' or $ext='xsd'">xml-validate</xsl:when>
    <xsl:when test="$mediatype = 'application/xslt+xml'">xml-transform</xsl:when>
    <!-- Main categories -->
    <xsl:when test="ends-with($mediatype, 'xml')">xml</xsl:when>
    <xsl:when test="starts-with($mediatype, 'text/')">text</xsl:when>
    <xsl:when test="starts-with($mediatype, 'image/')">image</xsl:when>
    <xsl:when test="starts-with($mediatype, 'audio/')">audio</xsl:when>
    <xsl:when test="starts-with($mediatype, 'video/')">video</xsl:when>
    <xsl:when test="$mediatype='folder'">folder</xsl:when>
    <!-- Fall back on  -->
    <xsl:otherwise><xsl:value-of select="replace($mediatype, '[^a-z0-9]', '-', 'i')"/></xsl:otherwise>
  </xsl:choose>
</xsl:function>

</xsl:stylesheet>