<?xml version="1.0"?>
<!--
  Templates used to display the files for a project.
 @author Jean-Baptiste Reure
 @version 22 October 2010
-->
<xsl:stylesheet version="2.0" xmlns="http://www.w3.org/1999/xhtml"
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:xs="http://www.w3.org/2001/XMLSchema"
                              xmlns:psf="http://www.pageseeder.com/function"
                              exclude-result-prefixes="xs psf">

  <xsl:import href="common.xsl"/>
    
  <!-- resource -->
  <xsl:template match="content[@name = 'resource']">
    <h3><xsl:value-of select="file/@name"/></h3>
    <div class="actions">
      <a href="{$context-path}/admin/resource?location={file/@path}&amp;download=true" class="download" title="Download file '{@name}'"><span class="icon"/>Download</a>
      <a href="#" data-reveal-id="file-history" class="history" title="File history" ><span class="icon"/>History</a>
    </div>
    <p><i>(<xsl:value-of select="file/@path"/>)</i></p>
    <xsl:choose>
      <xsl:when test="file/@type = 'folder'"><p>This file is a folder.</p></xsl:when>
      <xsl:otherwise><xsl:apply-templates select="file" mode="resource"/></xsl:otherwise>
    </xsl:choose>
    <p>Back to <a href="{$context-path}/admin/projects/{substring-before(substring-after(file/@path,'/WEB-INF/template/'), '/')}.html">files for this project</a></p>
    <script>var PublisherContext = '<xsl:value-of select="$context-path" />';</script>
    <script type="text/javascript" src="{$context-path}/js/lib/codemirror/codemirror-0.93.min.js"/>
  </xsl:template>

  <!-- File history -->
  <xsl:template match="content[@name = 'history']">
    <div id="file-history" class="lightbox reveal-modal medium" data-reveal="" aria-hidden="true" role="dialog">
      <div class="row inner">
        <div class="small-10 columns"><h1>File history</h1></div>
        <div class="small-2  columns text-right"><a class="close-reveal-modal" aria-label="Close" href="#">&#215;</a></div>
      </div>
      <div class="row inner"><div class="small-12 columns content">
        <table>
          <thead><tr>
            <th>Revision</th>
            <th>Size</th>
            <th>Modified</th>
            <th>Download</th>
          </tr></thead>
          <xsl:if test="empty(history/file)"><li>No history found.</li></xsl:if>
          <xsl:for-each select="history/file">
            <xsl:variable name="pos" select="position()"/>
            <tr>
              <td>
                <xsl:choose>
                  <xsl:when test="$pos = 1">Original</xsl:when>
                  <xsl:otherwise>Revision <xsl:value-of select="$pos - 1"/></xsl:otherwise>
                </xsl:choose>
              </td>
              <td><xsl:value-of select="psf:size(@length)"/></td>
              <td><xsl:value-of select="format-dateTime(@modified, '[D] [MNn,*-3], [Y] at [H01]:[m01]')"/></td>
              <td><a href="{$context-path}/admin/resource?location={@path}&amp;download=true"
                 class="download"  title="Download file '{@name}'"><span class="icon"/>Download</a></td>
            </tr>
          </xsl:for-each>
        </table>
      </div></div>
      <div class="row inner"><div class="small-12 columns ps-buttons text-center">
        <button class="primary close tiny">Close</button>
      </div></div>
    </div>
  </xsl:template>

<!-- DIFFERENT FILES ========================================================================== -->

<!--
  Processes the contents
-->
<xsl:template match="file[@text='true']" mode="resource">
  <xsl:variable name="readonly" select="@editable = 'false' or starts-with(@path, '/WEB-INF/template/default/')"/>
  <form action="{$context-path}/admin/resources/put.html" method="post">
    <div class="source" style="display: none">
      <textarea id="source-code">
        <xsl:if test="$readonly"><xsl:attribute name="readonly">readonly</xsl:attribute></xsl:if>
        <xsl:choose>
          <!-- Existing files -->
          <xsl:when test="not(@status = 'not-found')"><xsl:value-of select="text()"/></xsl:when>
          <xsl:when test="@media-type = 'application/xslt+xml'"><xsl:call-template name="new-xsl"/></xsl:when>
          <xsl:when test="@media-type = 'text/css'">
            <xsl:call-template name="new-css"><xsl:with-param name="config" select="substring-before(@name, '.css')"/></xsl:call-template>
          </xsl:when>
          <xsl:when test="@media-type = 'application/javascript'"><xsl:call-template name="new-js"/></xsl:when>
        </xsl:choose>
      </textarea>
    </div>
    <!-- Only if it is editable -->
    <input name="location" type="hidden" value="{@path}"/>
    <input name="type"     type="hidden" value="{psf:to-edit-type(@media-type)}"/>
    <xsl:choose>
      <xsl:when test="$readonly"><p class="note">Note: You cannot edit this file</p></xsl:when>
      <xsl:otherwise>
      <div class="buttons">
        <input type="submit" value="Save" class="save button disabled" disabled="disabled"/>
        <div id="save-result"></div>
      </div>
      </xsl:otherwise>
    </xsl:choose>
  </form>
</xsl:template>

<!--
  Processes the contents
-->
<xsl:template match="file[@text='false']" mode="resource">
  <xsl:choose>
    <xsl:when test="@media-type = 'image/png'" ><img src="{$context-path}/admin/resource?location={@path}"/></xsl:when>
    <xsl:when test="@media-type = 'image/gif'" ><img src="{$context-path}/admin/resource?location={@path}"/></xsl:when>
    <xsl:when test="@media-type = 'image/jpeg'"><img src="{$context-path}/admin/resource?location={@path}"/></xsl:when>
    <xsl:otherwise>
      <a href="{$context-path}/admin/resource?location={@path}&amp;download=true">Download file</a>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- FUNCTIONS ================================================================================= -->

<!-- 
  Returns the Brush to use 
  @param media-type The media type to map
-->
<xsl:function name="psf:to-edit-type">
  <xsl:param name="media-type"/>
  <xsl:choose>
    <xsl:when test="$media-type = 'application/schematron+xml'">xml</xsl:when>
    <xsl:when test="$media-type = 'application/stylesheet+xml'">xml</xsl:when>
    <xsl:when test="$media-type = 'application/xslt+xml'">xml</xsl:when>
    <xsl:when test="$media-type = 'application/xml'">xml</xsl:when>
    <xsl:when test="$media-type = 'application/json'">javascript</xsl:when>
    <xsl:when test="$media-type = 'application/javascript'">javascript</xsl:when>
    <xsl:when test="$media-type = 'text/properties'">properties</xsl:when>
    <xsl:when test="$media-type = 'text/css'">css</xsl:when>
    <xsl:when test="$media-type = 'text/xml'">xml</xsl:when>
  </xsl:choose>
</xsl:function>

<!-- Templates for creation ==================================================================== -->

<!-- Blank XSLT template -->
<xsl:template name="new-xsl">
<![CDATA[<?xml version="1.0"?>
<!--
  GENERATED TEMPLATES - ENTER COMMENT HERE.

  @see http://dev.pageseeder.com/html/api/servlets
  @see http://www.w3.org/TR/xslt20/
]]>
  @version <xsl:value-of select="format-date(current-date(), '[D] [MNn] [Y]')"/>
<![CDATA[-->
<xsl:stylesheet version="2.0"
                xmlns="http://www.w3.org/1999/xhtml"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:psf="http://www.pageseeder.com/function"
                exclude-result-prefixes="xs psf">

<!-- Main template -->
<xsl:template match="/content">
  <div id="ps-content-{@id}" class="content">
    <h2>[title of the page]</h2>

  </div>
</xsl:template>
 
</xsl:stylesheet>
]]>
</xsl:template>

<!-- Blank CSS template -->
<xsl:template name="new-css">
<xsl:param name="config"/>
/**
 * GENERATED STYLES - ENTER COMMENT HERE.
 *
 * @version <xsl:value-of select="format-date(current-date(), '[D] [MNn] [Y]')"/>
 */

/* If editing the CSS for a document, import standard document styles */
/* @import url(../../../weborganic/document/css/standard.css); */

/* Specific styles should be prefixed by '#ps-content-[page or block id]' */
#ps-content-<xsl:value-of select="$config"/> { }

</xsl:template>

<!-- Blank JavaScript template -->
<xsl:template name="new-js">
/**
 * GENERATED SCRIPT - ENTER COMMENT HERE.
 *
 * Note: The PAGESEEDER global object, jQuery and jQuery UI are always available
 * and loaded before this script.
 *
 * @see http://dev.pageseeder.com/html/api/javascript
 * @see https://developer.mozilla.org/en/JavaScript/Reference
 * @see http://api.jquery.com/ 
 *
 * @version <xsl:value-of select="format-date(current-date(), '[D] [MNn] [Y]')"/>
 */

// If using JSLint, you can declare the global dependencies below
/*global PAGESEEDER, $ */

// If using jQuery, execute code in a closure when the document is ready
$(document).ready(function() {

  // Enter code here...

});

</xsl:template>

<!-- FUNCTIONS ================================================================================= -->

<!-- 
  Returns a user friendly size specifying the unit (bytes, Kb, Mb)
  @param the length in bytes of the files.
 -->
<xsl:function name="psf:size">
  <!-- the length of the file in bytes -->
  <xsl:param name="length" as="xs:integer"/>
  <xsl:choose>
    <xsl:when test="$length gt 1000*1000"><xsl:value-of select="round-half-to-even($length div (1000*1000), 1)"/> MB</xsl:when>
    <xsl:when test="$length gt 1000"><xsl:value-of select="round-half-to-even($length div 1000, 1)"/> kB</xsl:when>
    <xsl:otherwise><xsl:value-of select="$length"/> bytes</xsl:otherwise>
  </xsl:choose>
</xsl:function>

</xsl:stylesheet>
