<?xml version="1.0"?>
<!--
  Templates used to display the files for a project.

  @author Jean-Baptiste Reure
  @author Christophe Lauret
  @version 17 March 2011
-->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:xs="http://www.w3.org/2001/XMLSchema"
                              xmlns:psf="http://www.pageseeder.com/function"
                              exclude-result-prefixes="xs psf">

  <xsl:import href="../common/mediatype.xsl"/>

  <xsl:variable name="context-path" select="/root/header/context-path"/>

  <!-- Main template -->
  <xsl:template match="content[@name = 'project']">
    <h2>Publisher files for project <xsl:value-of select="project/@name" /></h2>
    <div id="project-files" class="ps-tree-root">
      <ul class="ps-branch-wrapper">
        <xsl:apply-templates select="project/*" mode="projects" />
      </ul>
    </div>
    <script>
      var PublisherContext = '<xsl:value-of select="$context-path" />';
      var PublisherProject = '<xsl:value-of select="project/@name" />';
    </script>
    <script src="{$context-path}/js/plugins/jquery.pstree.js"></script>
    <xsl:if test="not(@name = 'default')">
      <script src="{$context-path}/js/plugins/jquery.psdropfile.js"></script>
    </xsl:if>
  </xsl:template>

  <!-- Drop box -->
  <xsl:template match="zone[@name='DROPBOX']" mode="projects">
    <li id="dropbox" class="ps-tree-node" data-location="{@path}">
      <div class="ps-tree-node-dropbox off {if (not(file)) then 'empty-folder' else ''}">
        <a href="#" class="toggle" title="Expand / Collapse"> </a>
        <span class="ps-tree-node-title" title="Dropbox">Dropbox</span>
      </div>
      <div class="ps-tree-node-content"> 
        <ul>
          <xsl:apply-templates select="descendant::file[@type='file']" mode="projects"><xsl:sort select="@type"/><xsl:sort select="@name"/></xsl:apply-templates>
        </ul>
      </div>
    </li>
  </xsl:template>

  <!-- templates or WO config -->
  <xsl:template match="zone[@name='TEMPLATE']" mode="projects">
    <li class="ps-tree-node">
      <div class="off ps-tree-node-folder">
        <a href="#" class="toggle" title="Expand / Collapse"> </a>
        <span class="ps-tree-node-title" title="templates">Template</span>
      </div>
      <div class="ps-tree-node-content" style="display: none"> 
        <ul>
          <xsl:apply-templates mode="projects"><xsl:sort select="@type"/><xsl:sort select="@name"/></xsl:apply-templates>
        </ul>
      </div>
    </li>
  </xsl:template>

  <!-- folders -->
  <xsl:template match="file[@type='folder']|file[@status='not-found']" mode="projects">
    <li class="ps-tree-node" data-location="{@path}">
      <div class="ps-tree-node-folder off {if (not(file)) then 'empty-folder' else ''} {if (@type='folder') then 'ps-droppable' else ''}">
        <a href="#" class="toggle" title="Expand / Collapse"> </a>
        <span class="ps-tree-node-title"><xsl:value-of select="@name"/></span>
        <span class="actions">
          <xsl:if test="not(ancestor::project/@name = 'default')">
            <a href="#delete"  class="delete"  title="Delete folder"><span>X</span></a>
            <a href="#upload"  class="upload"  title="Upload file into folder"><span>U</span></a>
            <a href="#create"  class="create"  title="Create file/folder in folder"><span>C</span></a>
          </xsl:if>
        </span>
      </div>
      <div class="ps-tree-node-content" style="display: none">
        <ul>
          <xsl:apply-templates mode="projects"><xsl:sort select="@type"/><xsl:sort select="@name"/></xsl:apply-templates>
        </ul>
      </div>
    </li>
  </xsl:template>

  <!-- files -->
  <xsl:template match="file[@type='file']" mode="projects">
    <li class="ps-tree-node" data-location="{@path}">
      <div class="ps-tree-node-file leaf{if (@overrides='true') then  'override' else ''}">
        <a class="ps-tree-node-title" title="File: {@name} {if (@overrides='true') then ' (overrides default)' else ''}" href="{psf:edit-resource(.)}">
          <span class="file">
            <xsl:sequence select="psf:file-to-mediaicon(.)"/>
            <xsl:text>&#160;</xsl:text>
            <xsl:value-of select="@name"/>
            <xsl:text>&#160;</xsl:text>
            <span class="size"><xsl:value-of select="psf:size(@length)"/></span>
          </span>
        </a>
        <span class="actions">
          <xsl:if test="not(ancestor::project/@name = 'default')">
            <a href="#delete"  class="delete"  title="Delete file"><span>X</span></a>
            <a href="#rename"  class="rename"  title="Rename file"><span>R</span></a>
            <a href="#history" class="history" title="Show file history"><span>H</span></a>
            <xsl:if test="@media-type = 'application/zip'">
              <a href="#import"  class="import"  title="Import pack"><span>I</span></a> 
            </xsl:if>
          </xsl:if>
          <a href="{$context-path}/admin/resource?location={@path}&amp;download=true" class="download" title="Download file"><span>D</span></a>
        </span>
        <time class="modified" datetime="{@modified}"><xsl:value-of select="format-dateTime(@modified, '[D] [MNn,*-3], [Y] at [H01]:[m01]')"/></time>
      </div>
    </li>
  </xsl:template>
  
  <!-- Utility functions ========================================================================= -->
  
  <!-- 
    Returns a user friendly size specifying the unit (bytes, Kb, Mb)
    @param the length in bytes of the files.
   -->
  <xsl:function name="psf:size">
    <!-- the length of the file in bytes -->
    <xsl:param name="length" as="xs:integer"/>
    <xsl:choose>
      <xsl:when test="$length gt 1000*1000"><xsl:value-of select="round-half-to-even($length div (1000*1000), 1)"/> MB</xsl:when>
      <xsl:when test="$length gt 1000"><xsl:value-of select="round-half-to-even($length div 1000, 1)"/> kB</xsl:when>
      <xsl:otherwise><xsl:value-of select="$length"/> bytes</xsl:otherwise>
    </xsl:choose>
  </xsl:function>

  <!--
    Returns the path to the specified resource based on the Media type.
    @param the file XML
  -->
  <xsl:function name="psf:edit-resource">
    <xsl:param name="file"/>
    <xsl:value-of select="$context-path" />
    <xsl:text>/admin/resources/get.html?location=</xsl:text>
    <!-- TODO encode directory name properly -->
    <xsl:value-of select="$file/@path" />
  </xsl:function>

</xsl:stylesheet>
