/**
 * An implementation of the Tree using the jQuery JavaScript Library v1.3.2
 * http://jquery.com/
 *
 * @author Christophe Lauret
 * @version 5.7900
 */

// Declare all global dependencies for JSLint
/*global jQuery, $, escape, unescape */

/**
 * Initialisation
 */

if (jQuery) {

 (function ($){

   $(document).ready(function(){
     $('.ps-tree-root').psTree();
   });

  $.extend($.fn, {

    psTree: function(o) {

      // Defaults
      if (!o) {o = {};}
      if (o.speed === undefined) {
        o.speed= 200;
      }

      // Attach the tree function
      $(this).each( function() {

        /**
         * Toggle the expand/collapse status of the tree.
         */
        var toggle = function toggle() {
          var $node = $(this).parents('.ps-tree-node:first');
          var $nodetitle = $node.children('.on, .off');
          var $nodecontent = $node.children('.ps-tree-node-content');
          // Expand
          if ($nodetitle.hasClass('off')) {
            var url = $(this).filter("[rel=subtree]").attr('href');
            if (url !== null) {
              showTree($nodecontent, url);
            } else {
              $nodecontent.slideDown(o.speed);
            }
          // Collapse
          } else {
            $nodecontent.slideUp(o.speed);
          }
          // update the cookie value
          updateCookie($(this));
          $nodetitle.toggleClass('on').toggleClass('off');
          return false;
        };

        /**
         * check if this tree is persistent and set cookie value if it is
         */
        function updateCookie(a) {
          // check for persistence
          var name = a.parents('div.ps-tree-root').attr('persistent');
          if (name === undefined) {return;}
          // get existing value
          var oldvalue = null;
          var start = document.cookie.indexOf(name+"=");
          var len = start+name.length+1;
          if (!start && name !== document.cookie.substring(0, name.length)) {oldvalue = null;}
          else if (start === -1) {oldvalue = null;}
          else {
            var end = document.cookie.indexOf(";",len);
            if (end === -1) {end = document.cookie.length;}
            oldvalue = decodeURIComponent(document.cookie.substring(len, end));
          }
          // compute new value
          var level = 0;
          a.parents('ul').each(function() {if ($(this).parents('.ps-tree-root')) {level++;}});
          var newvalue = level+':'+a.parents('li:first').attr('nodeid')+'|';
          if (a.parents('.ps-tree-node:first').children('.on, .off').hasClass('on')) { // closing it
            if (oldvalue === null || oldvalue === '') {return;}
            if (oldvalue.indexOf(newvalue) === 0) {newvalue = oldvalue.substring(newvalue.length);}
            else if (oldvalue.indexOf('|'+newvalue) !== -1) {
              newvalue = oldvalue.substring(0, oldvalue.indexOf('|'+newvalue)) + oldvalue.substring(oldvalue.indexOf('|'+newvalue) + newvalue.length);
            } else {newvalue = oldvalue;}
          } else if (oldvalue !== null) { // opening it
            if (oldvalue.indexOf(newvalue) !== 0 && oldvalue.indexOf('|'+newvalue) === -1) {newvalue += oldvalue;}
            else {newvalue = oldvalue;}
          }
          document.cookie = name + "=" + encodeURIComponent(newvalue).replace(/%2F/g, '/') + ";path=/";
        }

        /**
         * display branch: slide down if already loaded, otherwise loads it using ajax
         */
        function showTree(c, url) {
          // only load if tree has not been loaded for now.
          if ($(c).find('*').size() > 0) {
            $(c).slideDown(o.speed);
          } else {
            $(c).prev().addClass('tree-loading');
            loadContent(c, url);
          }
        }

        /**
         * @param url The URL to invoke.
         * @return The same URL with a timestamp to prevent caching.
         */
        function noCache(url) {
          return url + ((url.indexOf('?') >= 0)? '&' : '?')+ 't=' + (new Date()).getTime();
        }

        /**
         * enable links on tree
         * called on startup and when a new branch is loaded
         */
        function bindTree(t) {
          $(t).find('.ps-tree-node a.toggle').click(toggle);
//          $(t).find('.ps-tree-node > div:first-child > a.button').click(toggle);
        }

        /**
         * Load the content using AJAX and handles failed login.
         *
         * @param cont   The content HTML element
         * @param url    The URL for the content of the expanded the block.
         */
        function loadContent(cont, url) {
          // We load the children of the DIV block
          $(cont).load(noCache(url), '', function(responseText, textStatus, XMLHttpRequest) {
            var $content = $(this);
            switch (XMLHttpRequest.status) {
              case 500:
                $content.html('<div>A server error occurred.</div>');
                return;
              case 403:
                $content.html('<div>Access Denied</div>');
                return;
              case 401:
                // unauthorized, show login dialog window
                PAGESEEDER.Login.loginDialog(function() {
                  // succeeded, load content again
                  loadContent(cont, url);
                }, function() {
                  // user closed the logind dialog, show message in block
                  $content.html('<div>Access Denied</div>');
                });
                return;
              default:
                bindTree(cont);
                // check for change input
                $(cont).parents('.ps-tree-root').find('input.onchange').change();
                $(cont).prev().removeClass('tree-loading');
                $(cont).slideDown(o.speed);
            }
          });
        }

        // Initialise the tree
        bindTree($(this));
      });
    }

  });

 })(jQuery);
}